package models

import (
	"eta/eta_mini_crm/utils"
	"fmt"
	"strings"

	"github.com/beego/beego/v2/client/orm"
)

type ChartPermissionSearchKeyWordMapping struct {
	Id                int `orm:"pk" description:"id"`
	ChartPermissionId int `description:"品种id"`
	ClassifyId        int `description:"分类id"`
}

func GetClassifyIdsListById(chartPermissionId int) (classifyIds []int, err error) {
	o := orm.NewOrmUsingDB("rddp")
	sql := ` SELECT classify_id
		FROM chart_permission_search_key_word_mapping	
		WHERE chart_permission_id = ? `
	_, err = o.Raw(sql, chartPermissionId).QueryRows(&classifyIds)
	return
}

func GetChartPermissionIdsListByClassifyId(classifyId int) (chartPermissionIds []string, err error) {
	o := orm.NewOrmUsingDB("rddp")
	sql := ` SELECT chart_permission_id
		FROM chart_permission_search_key_word_mapping	
		WHERE classify_id = ? `
	_, err = o.Raw(sql, classifyId).QueryRows(&chartPermissionIds)
	return
}

func GetChartPermissionIdsListByClassifyIds(classifyId []int) (chartPermissionIds []int, err error) {
	if len(classifyId) == 0 {
		return
	}
	o := orm.NewOrmUsingDB("rddp")
	sql := ` SELECT chart_permission_id
		FROM chart_permission_search_key_word_mapping	
		WHERE classify_id = (%s) `
	sql = fmt.Sprintf(sql, utils.GetOrmReplaceHolder(len(classifyId)))

	_, err = o.Raw(sql, classifyId).QueryRows(&chartPermissionIds)
	return
}

func GetChartPermissionListByClassifyStrIds(classifyIds []string) (items []*ChartPermissionSearchKeyWordMapping, err error) {
	if len(classifyIds) == 0 {
		return
	}
	o := orm.NewOrmUsingDB("rddp")
	sql := ` SELECT * FROM chart_permission_search_key_word_mapping	WHERE 1=1 `
	var idsStr string
	if len(classifyIds) > 0 {
		idsStr = strings.Join(classifyIds, ",")
		sql += fmt.Sprintf(" AND classify_id in (%s) ", idsStr)
	}
	_, err = o.Raw(sql).QueryRows(&items)
	return
}

func GetChartPermissionListByClassifyIds(classifyIds []int) (items []*ChartPermissionSearchKeyWordMapping, err error) {
	if len(classifyIds) == 0 {
		return
	}
	o := orm.NewOrmUsingDB("rddp")
	sql := ` SELECT * FROM chart_permission_search_key_word_mapping	WHERE 1=1 `
	if len(classifyIds) > 0 {
		sql += fmt.Sprintf(" AND classify_id in (%s) ", utils.GetOrmReplaceHolder(len(classifyIds)))
	}
	_, err = o.Raw(sql, classifyIds).QueryRows(&items)
	return
}

func GetClassifyIdsListByIds(chartPermissionIds []int) (classifyIds []int, err error) {
	if len(chartPermissionIds) == 0 {
		return
	}
	o := orm.NewOrmUsingDB("rddp")
	sql := ` SELECT classify_id
		FROM chart_permission_search_key_word_mapping	
		WHERE 1=1 `
	if len(chartPermissionIds) > 0 {
		sql += fmt.Sprintf(" AND chart_permission_id in (%s) ", utils.GetOrmReplaceHolder(len(chartPermissionIds)))
	}
	_, err = o.Raw(sql, chartPermissionIds).QueryRows(&classifyIds)
	return
}
